#!/usr/bin/python3
import gi
import sys
import argparse
import managers.FileRestrictionManager as FileRestrictionManager
import ETAKisitActivator

# Privileged run check
if not FileRestrictionManager.check_user_privileged():
    sys.stderr.write("You are not privileged to run this script.\n")
    sys.exit(1)

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio  # noqa
from ui_gtk3.MainWindow import MainWindow  # noqa
from Logger import log  # noqa


class Main(Gtk.Application):
    def __init__(self):
        super().__init__(
            application_id="tr.org.pardus.eta-kisit",
            flags=Gio.ApplicationFlags.FLAGS_NONE,
        )

        self.window = None

    def do_activate(self):
        if self.window is None:
            self.window = MainWindow(self)

        self.window.show_ui()


# Argument Parsing
parser = argparse.ArgumentParser(description="Application & Domain restricting app.")

parser.add_argument(
    "--reload", action="store_true", help="Apply new profiles.json to the system."
)
parser.add_argument(
    "--user-login",
    action="store",
    help="Specify a user logged in. This disables network restrictions if user is a privileged user.",
)
parser.add_argument(
    "--user-logout",
    action="store",
    help="Specify a user logged out. This re-enables network restrictions if user is a unprivileged user.",
)
parser.add_argument(
    "--generate-applist",
    action="store_true",
    help='Write installed applications in "/var/lib/eta/eta-kisit/installed_applications.json"',
)
args = parser.parse_args()


if args.reload:
    # Deactive
    print("Reverting previous settings...")
    ETAKisitActivator.run_activator(["--unrestrict"])

    print("Applying new settings...")
    ETAKisitActivator.run_activator(["--restrict"])

    print("Done.")
elif args.user_login:
    import managers.ProfileManager as ProfileManager

    username = args.user_login

    log("logged username: {}".format(username))

    # Enable website restriction and exit if ogrenci
    if username == "ogrenci":
        if ETAKisitActivator.is_service_active():
            ETAKisitActivator.run_activator(["--enable-websites-restriction"])

        # Always restrict apps:
        ETAKisitActivator.run_activator(["--always-restricted-apps"])

        exit(0)
    else:
        # Disable website restrictions for normal user
        if ETAKisitActivator.is_service_active():
            ETAKisitActivator.run_activator(["--disable-websites-restriction"])

    profile_manager = ProfileManager.get_default()
    try:
        user_profile = profile_manager.get_profile(username)
        print("Profile found:", username)
        if user_profile.get_activate_on_startup():
            log(
                "'activate_on_startup' is true, activating profile: {}".format(username)
            )
            # Deactive
            print("Reverting previous settings...")
            ETAKisitActivator.run_activator(["--unrestrict"])

            profile_manager.set_current_profile(username)

            print("Applying new settings...")
            ETAKisitActivator.run_activator(["--restrict"])

            print("Done.")

    except KeyError:
        # No profile found, create a new one
        print("Profile not found, creating new one:", username)
        if username is not None:
            profile_manager.insert_default_profile(username)

elif args.user_logout:
    username = args.user_logout

    log("logout, username: {}".format(username))

    if ETAKisitActivator.is_service_active() and username == "ogrenci":
        ETAKisitActivator.run_activator(["--disable-websites-restriction"])

elif args.generate_applist:
    import managers.ApplicationManager as ApplicationManager

    ApplicationManager.save_all_applications()
    print(
        "Applications saved to {}".format(
            ApplicationManager.INSTALLED_APPLICATIONS_PATH
        )
    )
else:
    # Start GUI
    app = Main()
    app.run(sys.argv)
